@@ -5,40 +5,66 @@ use warnings;
 use Carp qw/croak/;
 use Scalar::Util qw/blessed/;
 use MooX::ReturnModifiers;
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 sub import {
     my ( $package, @import ) = @_;
-    
+
     my $target = caller;
-    my %modifiers = return_modifiers($target, [qw/has around with/]);
+    my %modifiers = return_modifiers( $target, [qw/has around with/] );
 
     my $variant = sub {
-        my ($name, %spec) = @_;
-       
-        $modifiers{has}->(
-            $name => _construct_attribute($name, %spec)
-        );
+        my ( $name, %spec ) = @_;
+
+        if ( my $builder = $spec{builder} ) {
+            my $build_name = $builder =~ m/^1$/ ? sprintf "_build_%s", $name : $builder;
+            $modifiers{around}->($build_name, sub { 
+                my ($orig, $self) = (shift, shift);
+                return $self->_given_when($self->$orig(@_), $spec{given}, $spec{when}, $name);
+            });
+        }
+
+        $modifiers{has}->( $name => _construct_attribute( $name, %spec ) );
 
     };
 
-    $target->can('_given_when') or $modifiers{with}->( 'MooX::VariantAttribute::Role' );
+    $target->can('_given_when')
+      or $modifiers{with}->('MooX::VariantAttribute::Role');
 
-    { no strict 'refs'; *{"${target}::variant"} = $variant;}
+    { no strict 'refs'; *{"${target}::variant"} = $variant; }
 
     return 1;
 }
 
 sub _construct_attribute {
-    my ($name, %spec) = @_;
+    my ( $name, %spec ) = @_;
 
     my $trigger = sub {
-        return $_[0]->_given_when($_[1], $spec{given}, $spec{when}, $name);
+        return $_[0]->_given_when( $_[1], $spec{given}, $spec{when}, $name );
     };
 
     return (
-        is => $spec{is} ? $spec{is} : 'rw',
+        is      => 'rw',
         trigger => $trigger,
+        (
+            $spec{default} ? (
+                default => sub {
+                    return $_[0]->_given_when(
+                        (
+                            ref $spec{default} eq 'CODE'
+                                ? $spec{default}->()
+                                : $spec{default}
+                        ),
+                        $spec{given},
+                        $spec{when},
+                        $name
+                    );
+                }
+            ) : ()
+        ),
+        (map {(
+            $spec{$_} ? ( $_ => $spec{$_} ) : ()
+        )} qw/builder lazy predicate clearer required reader writer/),
     );
 }
 
@@ -46,15 +72,15 @@ sub _construct_attribute {
 
 __END__
 
+=encoding utf8
+
 =head1 NAME
 
 MooX::VariantAttribute - a щ（ﾟДﾟщ）Attribute...
 
 =head1 VERSION
 
-Version 0.04
-
-=cut
+Version 0.05
 
 =head1 SYNOPSIS
 
@@ -157,23 +183,16 @@ with a trigger.
         }
     );
 
-By default variants are read-write, You can optionally set a variant to be read-only the normal way.
+These attributes have to be read-write as the trigger is used to set the final *stored* value.
 
-    variant 'one' => (
-        is => 'ro',
-        ...
-    )
-
-    my $foo = Backwards::World->new( one => 'world' );
-
-Variants should always have two key/value pairs, given and when, given accepts a code reference or L<Type::Tiny> object. When the code 
+The Variants should always have two key/value pairs, given and when, given accepts a code reference or L<Type::Tiny> object. When the code 
 reference is called two parameters are passed the first $self the second the $new value. Type::Tiny objects are called with 
 the $new value.
     
     given => sub { my $self = shift; ref $_[1] },
 
 When is well when it gets more complicated. *when* should always be an array reference of pairs. The first value is the *matching* value
-it can be a SCALAR, ARRAY, HASH maybe even a Object. The second value has to be a hash reference with two optional keys run and alias. 
+it can be a SCALAR, ARRAY, HASH maybe even an Object. The second value has to be a hash reference with two optional keys run and alias. 
 run should always be a code reference it will be passed $self, the $value returned from given, and the $new value. 
 
     package Backwards::World;
@@ -199,7 +218,9 @@ run should always be a code reference it will be passed $self, the $value return
         ],
     );
 
-    $object = Backwards::World->new( hello => 'one' );
+    ....
+    
+    my $object = Backwards::World->new( hello => 'one' );
 
     $object->hello # one;
 
@@ -212,6 +233,13 @@ run should always be a code reference it will be passed $self, the $value return
     $object->hello('seven'), # one;
     $object->hello, # one;
 
+You can also set the default value for a variant attribute using a *default* or a *builder*.
+
+    default => sub { one => two },
+    default => { one => two }, 
+
+    builder => 1,
+
 =head1 AUTHOR
 
 Robert Acock, C<< <thisusedtobeanemail at gmail.com> >>
@@ -251,10 +279,8 @@ L<http://search.cpan.org/dist/MooX-VariantAttribute/>
 
 =back
 
-
 =head1 ACKNOWLEDGEMENTS
 
-
 =head1 LICENSE AND COPYRIGHT
 
 Copyright 2017 Robert Acock.
@@ -295,7 +321,6 @@ CONTRIBUTOR WILL BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, OR
 CONSEQUENTIAL DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THE PACKAGE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-
 =cut
 
 1; # End of MooX::VariantAttribute
